@@ -19,6 +19,7 @@
  */
 
 #include "box_db_table_definition.h"
+#include <glom/frame_glom.h>
 #include <bakery/App/App_Gtk.h> //For util_bold_message().
 #include <glom/glom_postgres.h>
 #include "../../../config.h"
@@ -102,7 +103,7 @@ void Box_DB_Table_Definition::fill_field_row(const Gtk::TreeModel::iterator& ite
   m_AddDel.set_value(iter, m_colTitle, title);
 
   //Type:
-  Field::glom_field_type fieldType = Field::get_glom_type_for_gda_type(field->get_field_info().get_gdatype()); //Could be TYPE_INVALID if the gda type is not one of ours.
+  Field::glom_field_type fieldType = Field::get_glom_type_for_gda_type(field->get_field_info()->get_g_type()); //Could be TYPE_INVALID if the gda type is not one of ours.
 
   const Glib::ustring strType = Field::get_type_name_ui( fieldType );
   m_AddDel.set_value(iter, m_colType, strType);
@@ -175,8 +176,8 @@ void Box_DB_Table_Definition::on_adddel_add(const Gtk::TreeModel::iterator& row)
       field->set_title( Utils::title_from_string(name) ); //Start with a title that might be useful.
       field->set_glom_type(Field::TYPE_NUMERIC);
 
-      Gnome::Gda::FieldAttributes field_info = field->get_field_info();
-      field_info.set_gdatype( Field::get_gda_type_for_glom_type(Field::TYPE_NUMERIC) );
+      Glib::RefPtr<Gnome::Gda::Column> field_info = field->get_field_info();
+      field_info->set_g_type( Field::get_gda_type_for_glom_type(Field::TYPE_NUMERIC) );
       field->set_field_info(field_info);
 
       fill_field_row(row, field);
@@ -236,7 +237,7 @@ bool Box_DB_Table_Definition::check_field_change(const sharedptr<const Field>& f
 
   //If we are changing a non-glom type:
   //Refuse to edit field definitions that were not created by glom:
-  if(Field::get_glom_type_for_gda_type( field_old->get_field_info().get_gdatype() )  == Field::TYPE_INVALID)
+  if(Field::get_glom_type_for_gda_type( field_old->get_field_info()->get_g_type() )  == Field::TYPE_INVALID)
   {
     Gtk::MessageDialog dialog(Bakery::App_Gtk::util_bold_message(_("Invalid database structure")), true);
     dialog.set_secondary_text(_("This database field was created or edited outside of Glom. It has a data type that is not supported by Glom. Your system administrator may be able to correct this."));
@@ -250,7 +251,7 @@ bool Box_DB_Table_Definition::check_field_change(const sharedptr<const Field>& f
   else
   {
     //Refuse to set a second primary key:
-    bool bcontinue = true;
+    //bool bcontinue = true;
     if(field_new->get_primary_key() && !field_old->get_primary_key()) //Was the primary key column checked?
     {
       //Is there an existing primary key?
@@ -323,6 +324,10 @@ void Box_DB_Table_Definition::on_adddel_edit(const Gtk::TreeModel::iterator& row
 
   //m_pDialog->set_modified(false); //Disable [Apply] at start.
 
+  Gtk::Window* parent_window = get_app_window();
+  if(parent_window)
+    m_pDialog->set_transient_for(*parent_window);
+
   m_pDialog->show();
 }
 
@@ -360,11 +365,11 @@ sharedptr<Field> Box_DB_Table_Definition::get_field_definition(const Gtk::TreeMo
   sharedptr<const Field> field_temp = get_fields_for_table_one_field(m_table_name, strFieldNameBeforeEdit);
   if(field_temp)
   {
-    Gnome::Gda::FieldAttributes fieldInfo = field_temp->get_field_info();
+    Glib::RefPtr<Gnome::Gda::Column> fieldInfo = field_temp->get_field_info();
 
     //Name:
     const Glib::ustring name = m_AddDel.get_value(row, m_colName);
-    fieldInfo.set_name(name);
+    fieldInfo->set_name(name);
 
     //Title:
     const Glib::ustring title = m_AddDel.get_value(row, m_colTitle);
@@ -374,17 +379,17 @@ sharedptr<Field> Box_DB_Table_Definition::get_field_definition(const Gtk::TreeMo
     const Glib::ustring& strType = m_AddDel.get_value(row, m_colType);
 
     const Field::glom_field_type glom_type =  Field::get_type_for_ui_name(strType);
-    Gnome::Gda::ValueType fieldType = Field::get_gda_type_for_glom_type(glom_type);
+    GType fieldType = Field::get_gda_type_for_glom_type(glom_type);
 
     //Unique:
     const bool bUnique = m_AddDel.get_value_as_bool(row, m_colUnique);
-    fieldInfo.set_unique_key(bUnique);
+    fieldInfo->set_unique_key(bUnique);
 
     //Primary Key:
     const bool bPrimaryKey = m_AddDel.get_value_as_bool(row, m_colPrimaryKey);
-    fieldInfo.set_primary_key(bPrimaryKey);
+    fieldInfo->set_primary_key(bPrimaryKey);
 
-    fieldInfo.set_gdatype(fieldType);
+    fieldInfo->set_g_type(fieldType);
 
     //Put it together:
     fieldResult->set_field_info(fieldInfo);
@@ -417,9 +422,48 @@ sharedptr<Field> Box_DB_Table_Definition::change_definition(const sharedptr<cons
 {
   Bakery::BusyCursor busy_cursor(get_app_window());
 
-  //DB field defintion:
+  //DB field definition:
 
   sharedptr<Field> result;
+  
+  if(!fieldOld || !field)
+    return result;
+
+  if(fieldOld->get_primary_key() != field->get_primary_key())
+  {
+    //If setting the primarykeyness, check whether this is likely to succeed:
+    if(!fieldOld->get_primary_key() && field->get_primary_key())
+    {
+      //Check for nulls:
+      if(field_has_null_values(fieldOld)) //Use the fieldOld because we only use the name, and we want to check the _existing_ field:
+      {
+        Gtk::Window* window = const_cast<Gtk::Window*>(get_app_window());
+        if(window)
+          Frame_Glom::show_ok_dialog(_("Field contains empty values."), _("The field may not yet be used as a primary key because it contains empty values."), *window, Gtk::MESSAGE_WARNING);
+
+          //TODO: Offer to fill it in with generated ID numbers? That could give strange results if the existing values are human-readable.
+
+        return glom_sharedptr_clone(fieldOld); //No changes were made.
+      }
+
+      //Check that the values are unique:
+      if(field_has_non_unique_values(fieldOld)) //Use the fieldOld because we only use the name, and we want to check the _existing_ field:
+      {
+        Gtk::Window* window = const_cast<Gtk::Window*>(get_app_window());
+        if(window)
+          Frame_Glom::show_ok_dialog(_("Field contains non-unique values."), _("The field may not yet be used as a primary key because it contains values that are not unique."), *window, Gtk::MESSAGE_WARNING);
+
+        return glom_sharedptr_clone(fieldOld); //No changes were made.
+      }
+
+   
+    }
+
+    //Forget the remembered currently-viewed primary key value, 
+    //because it will be useless with a different field as the primary key, or with no field as primary key:
+    Document_Glom* document = get_document();
+    document->forget_layout_record_viewed(m_table_name);
+  }
 
   try
   {
@@ -501,11 +545,11 @@ void Box_DB_Table_Definition::fill_fields()
 
 sharedptr<Field> Box_DB_Table_Definition::postgres_change_column(const sharedptr<const Field>& field_old, const sharedptr<const Field>& field)
 {
-  const Gnome::Gda::FieldAttributes field_info = field->get_field_info();
-  const Gnome::Gda::FieldAttributes field_info_old = field_old->get_field_info();
+  const Glib::RefPtr<Gnome::Gda::Column> field_info = field->get_field_info();
+  const Glib::RefPtr<Gnome::Gda::Column> field_info_old = field_old->get_field_info();
 
   //If the underlying data type has changed:
-  if( field_info.get_gdatype() != field_info_old.get_gdatype() )
+  if(field_info->get_g_type() != field_info_old->get_g_type() )
   {
     postgres_change_column_type(field_old, field); //This will also change everything else at the same time.
     return glom_sharedptr_clone(field);
@@ -518,9 +562,13 @@ sharedptr<Field> Box_DB_Table_Definition::postgres_change_column(const sharedptr
 }
 
 
-void  Box_DB_Table_Definition::postgres_change_column_type(const sharedptr<const Field>& field_old, const sharedptr<const Field>& field)
+void Box_DB_Table_Definition::postgres_change_column_type(const sharedptr<const Field>& field_old, const sharedptr<const Field>& field)
 {
-  sharedptr<SharedConnection> sharedconnection = connect_to_server(get_app_window());
+  Gtk::Window* parent_window = get_app_window();
+  if(!parent_window)
+    return;
+
+  sharedptr<SharedConnection> sharedconnection = connect_to_server(parent_window);
   if(sharedconnection)
   {
     Glib::RefPtr<Gnome::Gda::Connection> gda_connection = sharedconnection->get_gda_connection();
@@ -528,7 +576,7 @@ void  Box_DB_Table_Definition::postgres_change_column_type(const sharedptr<const
     bool new_column_created = false;
 
     //If the datatype has changed:
-    if(field->get_field_info().get_gdatype() != field_old->get_field_info().get_gdatype())
+    if(field->get_field_info()->get_g_type() != field_old->get_field_info()->get_g_type())
     {
       //We have to create a new table, and move the data across:
       //See http://www.postgresql.org/docs/faqs/FAQ.html#4.4
@@ -538,10 +586,11 @@ void  Box_DB_Table_Definition::postgres_change_column_type(const sharedptr<const
       //    ALTER TABLE tab DROP COLUMN old_col;
       //    COMMIT;
 
-      Glib::RefPtr<Gnome::Gda::Transaction> transaction = Gnome::Gda::Transaction::create("glom_transaction_change_field_type");
-      const bool test = gda_connection->begin_transaction(transaction);
+      const Glib::ustring transaction_name = "glom_transaction_change_field_type"; 
+      const bool test = gda_connection->begin_transaction(transaction_name, Gnome::Gda::TRANSACTION_ISOLATION_UNKNOWN); // TODO: I am absolutely not sure what this transaction isolation does
       if(test)
       {
+        //Glib::RefPtr<Gnome::Gda::TransactionStatus> transaction = gda_connection->get_transaction_status();
         //TODO: Warn about a delay, and possible loss of precision, before actually doing this.
         //TODO: Try to use a unique name for the temp column:
 
@@ -644,7 +693,7 @@ void  Box_DB_Table_Definition::postgres_change_column_type(const sharedptr<const
             datamodel = query_execute( "ALTER TABLE \"" + m_table_name + "\" RENAME COLUMN \"" + fieldTemp->get_name() + "\" TO \"" + field->get_name() + "\"", get_app_window());
             if(datamodel)
             {
-              const bool test = gda_connection->commit_transaction(transaction);
+              const bool test = gda_connection->commit_transaction(transaction_name);
               if(!test)
               {
                 handle_error();
@@ -657,11 +706,71 @@ void  Box_DB_Table_Definition::postgres_change_column_type(const sharedptr<const
       }   //TODO: Abandon the transaction if something failed.
     }  /// If the datatype has changed:
 
-    if(!new_column_created) //We don't need to change anything else if everything was alrady correctly set as a new column:
+    if(!new_column_created) //We don't need to change anything else if everything was already correctly set as a new column:
       GlomPostgres::postgres_change_column_extras(m_table_name, field_old, field);
   }
 }
 
+bool Box_DB_Table_Definition::field_has_null_values(const sharedptr<const Field>& field)
+{
+  //Note that "= Null" doesn't work, though it doesn't error either.
+  //Note also that SELECT COUNT always returns 0 if all the values are NULL, so we can't use that to be more efficient.
+  const Glib::ustring sql_query = "SELECT \"" + field->get_name() + "\" FROM \"" + m_table_name + "\" WHERE \"" + m_table_name + "\".\"" + field->get_name() + "\" IS NULL ";
+  //std::cout << "sql_query: " << sql_query << std::endl;
+
+  long null_count = 0;
+  Glib::RefPtr<Gnome::Gda::DataModel> datamodel = query_execute(sql_query, get_app_window());
+  if(datamodel && datamodel->get_n_rows() && datamodel->get_n_columns())
+  {
+    null_count = datamodel->get_n_rows();
+    //std::cout << "debug: null_count = " << null_count << std::endl;
+  }
+  else
+  {
+    g_warning("Box_DB_Table_Definition::field_has_null_values(): SELECT COUNT() failed.");
+  }
+
+   return null_count > 0; 
+}
+
+bool Box_DB_Table_Definition::field_has_non_unique_values(const sharedptr<const Field>& field)
+{
+  long count_distinct = 0;
+  long count_all = 0;
+
+  //Count the distint rows:
+  const Glib::ustring sql_query_distinct = "SELECT DISTINCT \"" + field->get_name() + "\" FROM \"" + m_table_name + "\"";
+  
+  Glib::RefPtr<Gnome::Gda::DataModel> datamodel = query_execute(sql_query_distinct, get_app_window());
+  if(datamodel)
+  {
+    count_distinct = datamodel->get_n_rows();
+    //std::cout << "debug: null_count = " << null_count << std::endl;
+  }
+  else
+  {
+    g_warning("Box_DB_Table_Definition::field_has_non_unique_values(): SELECT COUNT() failed.");
+  }
+
+  //Count all rows, to compare. TODO_performance: Is there a more efficient way to do this? Maybe count(*), which apparently doesn't ignore NULL rows like count(somefield) would.
+  const Glib::ustring sql_query_all = "SELECT \"" + field->get_name() + "\" FROM \"" + m_table_name + "\"";
+  
+  datamodel = query_execute(sql_query_all, get_app_window());
+  if(datamodel)
+  {
+    count_all = datamodel->get_n_rows();
+    //std::cout << "debug: null_count = " << null_count << std::endl;
+  }
+  else
+  {
+    g_warning("Box_DB_Table_Definition::field_has_non_unique_values(): SELECT COUNT() failed.");
+  }
+
+
+  return count_distinct != count_all;
+}
+
+
 } //namespace Glom
 
 
