@@ -20,6 +20,7 @@
 
 //We need to include this before anything else, to avoid redefinitions:
 #include <glom/libglom/python_embed/py_glom_record.h>
+#include <glom/libglom/python_embed/pygdavalue_conversions.h>
 
 #define NO_IMPORT_PYGTK //To avoid a multiple definition in pygtk.
 #include <pygtk/pygtk.h> //For the PyGObject and PyGBoxed struct definitions.
@@ -32,6 +33,9 @@
 #include <glom/libglom/data_structure/glomconversions.h>
 #include <list>
 #include <glib.h> //For g_warning().
+#include <glibmm/i18n.h>
+
+#include <gtkmm/messagedialog.h>
 
 namespace Glom
 {
@@ -59,19 +63,86 @@ std::list<Glib::ustring> ustring_tokenize(const Glib::ustring& msg, const Glib::
   return result;
 }
 
+// Use this for errors not (directly) caused by the user
 void HandlePythonError()
 {
   if(PyErr_Occurred())
     PyErr_Print();
 }
 
-void glom_execute_python_function_implementation(const Glib::ustring& func_impl, const type_map_fields& field_values, Document_Glom* pDocument, const Glib::ustring& table_name)
+// Show python coding errors of the user
+void ShowTrace()
 {
-  glom_evaluate_python_function_implementation(Field::TYPE_TEXT, func_impl, field_values, pDocument, table_name);
+  // Python equivilant:
+  // import traceback, sys
+  // return "".join(traceback.format_exception(sys.exc_type,
+  //    sys.exc_value, sys.exc_traceback))
+
+  PyObject *type, *value, *traceback;
+
+  PyErr_Fetch(&type, &value, &traceback);
+  
+  if (!traceback)
+  {
+    std::cerr << "traceback = 0" << std::endl;
+  }
+
+  PyObject *tracebackModule = PyImport_ImportModule("traceback");
+  gchar* chrRetval = 0;
+  if (tracebackModule != NULL)
+  {
+      PyObject* tbList = PyObject_CallMethod(
+          tracebackModule,
+          "format_exception",
+          "OOO",
+          type,
+          value == NULL ? Py_None : value,
+          traceback == NULL ? Py_None : traceback);
+      
+      if (!tbList)
+      {
+        std::cerr << "format_exception failed" << std::endl;
+        return;
+      }
+
+      PyObject* emptyString = PyString_FromString("");
+      PyObject* strRetval = PyObject_CallMethod(emptyString, "join",
+            "O", tbList);
+      if (strRetval)
+        chrRetval = g_strdup(PyString_AsString(strRetval));
+    
+      Py_DECREF(tbList);
+      Py_DECREF(emptyString);
+      Py_DECREF(strRetval);
+      Py_DECREF(tracebackModule);
+    }
+    else
+    {
+        std::cerr << "Unable to import traceback module." << std::endl;
+        
+    }
+
+    Py_DECREF(type);
+    Py_XDECREF(value);
+    Py_XDECREF(traceback);
+  
+  if (chrRetval)
+  {
+    Glib::ustring message = _("Python Error: \n");
+    message += chrRetval;
+    Gtk::MessageDialog dialog(message, false, Gtk::MESSAGE_ERROR);
+    dialog.run();
+  }
+  g_free(chrRetval);
+}
+
+void glom_execute_python_function_implementation(const Glib::ustring& func_impl, const type_map_fields& field_values, Document_Glom* pDocument, const Glib::ustring& table_name, const Glib::RefPtr<Gnome::Gda::Connection>& opened_connection)
+{
+  glom_evaluate_python_function_implementation(Field::TYPE_TEXT, func_impl, field_values, pDocument, table_name, opened_connection);
 }
 
 Gnome::Gda::Value glom_evaluate_python_function_implementation(Field::glom_field_type result_type, const Glib::ustring& func_impl,
-    const type_map_fields& field_values, Document_Glom* pDocument, const Glib::ustring& table_name)
+    const type_map_fields& field_values, Document_Glom* pDocument, const Glib::ustring& table_name, const Glib::RefPtr<Gnome::Gda::Connection>& opened_connection)
 {
   //std::cout << "glom_evaluate_python_function_implementation()" << std::endl;
   //for(type_map_fields::const_iterator iter = field_values.begin(); iter != field_values.end(); ++iter)
@@ -98,13 +169,41 @@ Gnome::Gda::Value glom_evaluate_python_function_implementation(Field::glom_field
 
   //prefix the def line:
   const Glib::ustring func_name = "glom_calc_field_value";
-  //TODO: When pygda packages are available: func_def = "def " + func_name + "(record):\n  import gda\n" + func_def;
   func_def = "def " + func_name + "(record):\n  import glom\n  import gda\n" + func_def;
   //We did this in main(): Py_Initialize();
 
   PyObject* pMain = PyImport_AddModule("__main__");
   PyObject* pDict = PyModule_GetDict(pMain);
 
+
+  //Allow the function to import from our script library:
+  if (pDocument)
+  {
+    const std::vector<Glib::ustring> module_names = pDocument->get_library_module_names();
+    for(std::vector<Glib::ustring>::const_iterator iter = module_names.begin(); iter != module_names.end(); ++iter)
+    {
+      const Glib::ustring name = *iter;
+      const Glib::ustring script = pDocument->get_library_module(name);
+      if(!name.empty() && !script.empty())
+      {
+        PyObject* objectCompiled = Py_CompileString(script.c_str(), name.c_str() /* "filename", for debugging info */,  Py_file_input /* "start token" for multiple lines of code. */); //Returns a reference.
+  
+        if(!objectCompiled)
+          HandlePythonError();
+
+        PyObject* pObject = PyImport_ExecCodeModule(const_cast<char*>(name.c_str()), objectCompiled); //Returns a reference. //This should make it importable.
+
+        if(!pObject)
+         HandlePythonError();
+
+        Py_DECREF(pObject);
+        Py_DECREF(objectCompiled);
+        //TODO: When do these stop being importable? Should we unload them somehow later?
+      }
+    }
+  }
+
+
   PyObject* module_glom = PyImport_ImportModule("glom");
   if(!module_glom)
     g_warning("Could not import python glom module.");
@@ -120,10 +219,13 @@ Gnome::Gda::Value glom_evaluate_python_function_implementation(Field::glom_field
   if(!module_gda)
     g_warning("Could not import python gda module.");
 
+  // Gda.Value does not exist anymore in pygda-3.0
+#if 0
   PyObject* module_gda_dict = PyModule_GetDict(module_gda);
   PyObject* pyTypeGdaValue = PyDict_GetItemString(module_gda_dict, "Value"); //TODO: Unref this?
   if(!pyTypeGdaValue || !PyType_Check(pyTypeGdaValue))
     g_warning("Could not get gda.Value from gda_module.");
+#endif
 
 
   //Create the function definition:
@@ -134,16 +236,21 @@ Gnome::Gda::Value glom_evaluate_python_function_implementation(Field::glom_field
     pyValue = 0;
   }
   else
-    HandlePythonError();
+  {
+    ShowTrace();
+  }
 
   //Call the function:
   {
     PyObject* pFunc = PyDict_GetItemString(pDict, func_name.c_str()); //The result is borrowed, so should not be dereferenced.
     if(!pFunc)
+    {
       HandlePythonError();
+    }
 
     if(!PyCallable_Check(pFunc))
     {
+      HandlePythonError();
       g_warning("pFunc is not callable.");
       return valueResult;
     }
@@ -161,7 +268,7 @@ Gnome::Gda::Value glom_evaluate_python_function_implementation(Field::glom_field
       Py_INCREF(pParam); //TODO: As I understand it, PyObject_New() should return a new reference, so this should not be necessary.
 
       //Fill the record's details:
-      PyGlomRecord_SetFields(pParam, field_values, pDocument, table_name);
+      PyGlomRecord_SetFields(pParam, field_values, pDocument, table_name, opened_connection);
 
       PyObject* pArgs = PyTuple_New(1);
       PyTuple_SetItem(pArgs, 0, (PyObject*)pParam); //The pParam reference is taken by PyTuple_SetItem().
@@ -180,35 +287,16 @@ Gnome::Gda::Value glom_evaluate_python_function_implementation(Field::glom_field
         //Deal with the various possible return types:
         bool object_is_gda_value = false;
 
-        //This is a hack - see below:
-        //if( strcmp(pyResult->ob_type->tp_name, "gda.Value") == 0 )
-        //  object_is_gda_value = true;
+        GValue value = { 0 };
+        const int test = pygda_value_from_pyobject(&value, pyResult);
 
-        //g_warning("debug: pyResult->ob_type->tp_name=%s", pyResult->ob_type->tp_name);
-        //g_warning("debug: pyTypeGdaValue->tp_name=%s", ((PyTypeObject*)pyTypeGdaValue)->tp_name);
+        if(test == 0) //-1 means error.
+	  object_is_gda_value = true;
 
-        const int test = PyType_IsSubtype(pyResult->ob_type, (PyTypeObject*)pyTypeGdaValue);
-        if(test == -1) //-1 means error.
-          HandlePythonError();
-
-        if(test == 1) // 0 means false, -1 means error.
-          object_is_gda_value = true;
-
-        if(object_is_gda_value)
+        if(object_is_gda_value && G_IS_VALUE(&value))
         {
-          //Cast it to the "derived" struct type:
-          //All boxed types are wrapped with PyGBoxed in pygtk, without their own special structs,
-          //and GValue is a boxed-type.
-          PyGBoxed* pygtkobject = (PyGBoxed*)pyResult;
-          if(pygtkobject)
-          {
-            GdaValue* cboxed = (GdaValue*)pygtkobject->boxed;
-            if(cboxed)
-              valueResult = Glib::wrap(cboxed, true /* take_copy */);
-            g_warning("PyGBoxed::boxed is null");
-          }
-          else
-            g_warning("PyGBoxed is null");
+	  valueResult = Gnome::Gda::Value(&value);
+	  g_value_unset(&value);
         }
         else
         {
@@ -236,10 +324,10 @@ Gnome::Gda::Value glom_evaluate_python_function_implementation(Field::glom_field
                 valueResult = Conversions::parse_value(result_type, pchResult, success, true /* iso_format */);
               }
               else
-                g_warning("pchResult is null");
+                HandlePythonError();
             }
             else
-              g_warning("PyString_Check returned false");
+              HandlePythonError();
           }
         }
 
